@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2021, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -42,7 +42,7 @@ else:
     xmlschema.XMLSchema.meta_schema.build()
 
 from elementpath import *
-from elementpath.namespaces import XSD_NAMESPACE, XSI_NAMESPACE
+from elementpath.namespaces import XSI_NAMESPACE
 from elementpath.datatypes import DateTime10, DateTime, Date, Time, Timezone, \
     DayTimeDuration, YearMonthDuration, QName, UntypedAtomic
 
